#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "htmlcallbacks.h"
#include "htmlinterface.h"
#include "htmlbuffer.h"

#include "support.h"

#include <string.h>

GtkTextBuffer * buffer;
GtkTextView * textview;
GtkTextBuffer * buffer_output;

void
on_dialoglite_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
}

gboolean
on_dialoglite_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	//gtk_main_quit ();
	return FALSE;
}

void
setup_buffer (GtkWidget       *widget, gchar *text, gboolean *debug)
{
	textview = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(widget), "textview1"));
	buffer = create_buffer (textview);

	GtkWidget * textview_Output = lookup_widget(GTK_WIDGET(widget), "textview_Output");
	GtkTextTagTable * tagtable_output =  gtk_text_tag_table_new();
	buffer_output = gtk_text_buffer_new (tagtable_output);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(textview_Output) , buffer_output);

	if(!debug)
	{
		GtkWidget * vbox_debug = lookup_widget(GTK_WIDGET(widget), "vbox_debug");
		gtk_widget_hide(vbox_debug);
	}
	set_XHTML(buffer, text);
}
 
void
on_toolbtnCut_clicked                  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gtk_text_buffer_cut_clipboard(buffer, gtk_clipboard_get(GDK_NONE), TRUE);
}


void
on_toolbtnCopy_clicked                 (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gtk_text_buffer_copy_clipboard(buffer, gtk_clipboard_get(GDK_NONE));
}


void
on_toolbtnPaste_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gtk_text_buffer_paste_clipboard(buffer, gtk_clipboard_get(GDK_NONE), NULL, TRUE);
}


void
on_toolbtnDeleteText_clicked               (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gtk_text_buffer_delete_selection(buffer, TRUE, TRUE);
}


void
on_toolbtnClear_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkTextIter begin, end;
	gtk_text_buffer_get_start_iter (buffer, &begin);
	gtk_text_buffer_get_end_iter (buffer, &end);
	gtk_text_buffer_remove_all_tags(buffer, &begin, &end);
}

void
on_toolbtnUndo_clicked                 (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	Undo(buffer);
}


void
on_toolbtnRedo_clicked                 (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	Redo(buffer);
}

void
on_toolbtnBold_clicked                 (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Bold(buffer);
}


void
on_toolbtnItalic_clicked               (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Italic(buffer);
}


void
on_toolbtnUnder_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Underline(buffer);
}


void
on_toolbtnStrike_clicked               (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Strikethrough(buffer); //Strikethrough
}


void
on_toolbtnSuper_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Superscript(buffer);
}


void
on_toolbtnSub_clicked                  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	toggle_Subscript(buffer);
}


void
on_okbutton1_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_toolbtnColor_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	open_color_dialog(buffer);
}

void
on_toolbuttonSymbol_clicked            (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	open_symbol_dialog(buffer);
}


void
on_btn_ExportHtml_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkTextIter begin, end;
	gtk_text_buffer_get_bounds (buffer, &begin, &end);

	//gchar * xml =  gtk_text_buffer_get_text(buffer, &begin, &end, TRUE);
	gchar * html = WriteXHTML(buffer, "filename");
	gtk_text_buffer_set_text (buffer_output, html, -1);
}


void
on_btn_ImportHtml_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
//Get xml from output textview
GtkTextIter begin, end;
 gtk_text_buffer_get_bounds  (buffer_output, &begin, &end);

gchar * xml =  gtk_text_buffer_get_text(buffer_output, &begin, &end, TRUE);
printf("xml:\n%s\n", xml);
import_XHTML (buffer, xml);
}

gchar * html_response = NULL;

void
on_dialoglite_response                 (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
	if (response_id == GTK_RESPONSE_OK)
	{
		html_response = WriteXHTML(buffer, "filename");
		printf("return XML:%s", html_response);	
	}
	gtk_widget_hide(GTK_WIDGET(dialog));
}

void
on_combobox_unicodetype_changed        (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	GtkWidget *tableSymbol = lookup_widget(GTK_WIDGET(combobox), "tableSymbol");
		//rebuild table
	gtk_container_foreach (GTK_CONTAINER (tableSymbol),
			       (GtkCallback) gtk_widget_destroy,
			       NULL);
	
	int n, r, c;
	r=0;
	c=0;

	int start_char = 0;
	int end_char = 0;
	
	int chosen_unicode_type = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));
	//char * chosen_unicode_text=  gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
	printf("Chosen Unicode Type:%d", chosen_unicode_type);
	switch (chosen_unicode_type)
	{
		case (0): //Arabic  U+0600  -   U+06FF  (1536-1791)
			start_char = 1536;
			end_char = 1791;
			break;
		case (1): //Arrows Unicode U+2190 ? U+21FF   (8592-8703)
			start_char = 8592;
			end_char = 8703;
			break;
//		case (1): //Arrows-A Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
//		case (1): //Arrows-B Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
		case (2): //Basic Latin Unicode U+0000 ? U+007F   (0-127)
			start_char = 0;
			end_char = 127;
			break;
		case (3): //Currency Symbols Unicode U+20A0 ? U+20CF   (8352-8399)
			start_char = 0;
			end_char = 127;
			break;
		case (4): //General Punctuation Unicode U+2000 ? U+206F   (8192-8303)
			start_char = 8192;
			end_char = 8303;
			break;
		case (5): //Geometric Shapes Unicode U+25A0 ? U+25FF   (9632-9727)
			start_char = 9632;
			end_char = 9727;
			break;
		case (6): //Greek Unicode U+0370 ? U+03FF   (880-1023)
			start_char = 880;
			end_char = 1023;
			break;
		case (7): //Latin-1 Supplement Unicode U+0080 ? U+00FF   (128-255)
			start_char = 128;
			end_char = 255;
			break;
		case (8): //Latin Extended-A Unicode U+0100 ? U+017F   (256-383)
			start_char = 256;
			end_char = 383;
			break;
		case (9): //Latin Extended-B Unicode U+0180 ? U+024F   (384-591)
			start_char = 384;
			end_char = 591;
			break;
		case (10): //Letterlike Symbols Unicode U+2100 ? U+214F   (8448-8527)
			start_char = 8448;
			end_char = 8527;
			break;
		case (11): //Mathematical Operators Unicode U+2200 ? U+22FF   (8704-8959)
			start_char = 8704;
			end_char = 8959;
			break;
		case (12): //Miscellaneous Technical Unicode U+2300 ? U+23FF   (8960-9215)
			start_char = 8960;
			end_char = 9215;
			break;
		case (13): //Miscellaneous Symbols Unicode U+2600 ? U+26FF   (9728-9983)
			start_char = 9728;
			end_char = 9983;
			break;
		case (14): //Spacing Modifier Letters Unicode U+02B0 ? U+02FF   (688-767)
			start_char = 688;
			end_char = 767;
			break;
		default:
			printf("other");
	}
	for(n=start_char; n<end_char; n++)
{
		static gchar buf[10];
		int len;
		len = g_unichar_to_utf8 (n, buf);
		buf[len] = '\0';

		//printf("n:%d, buf:%s\n", n, buf);
 		const gchar *end;
		if(g_utf8_validate (buf,  -1, &end) && !g_unichar_isspace (buf[0]))
{
			if (strcmp(buf, ""))
{
 				GtkWidget *btnChar;
 				btnChar = gtk_button_new_with_label (buf);
 				gtk_button_set_relief(GTK_BUTTON(btnChar), GTK_RELIEF_NONE);
 				gtk_table_attach_defaults   (GTK_TABLE(tableSymbol),
 					     btnChar,
 					     c,
 					     c+1,
 					     r,
 					     r+1);
  				g_signal_connect ((gpointer) btnChar, "enter",
                    			G_CALLBACK (on_button_Symbol_activate),
                    			NULL);
  				g_signal_connect ((gpointer) btnChar, "clicked",
                    			G_CALLBACK (on_button_Symbol_clicked),
                    			NULL);
				c += 1;
				if (c > 10)
{
					c = 0;
					r += 1;
}
 				gtk_widget_show(btnChar);
}
}
		else
{
			//g_print(" invalid\n");
}
}
}

